xquery version "3.1";

(:~
 : This module contains the transformation and complementary functions of the
 : interactive view for the KMG website.
 :
 : @author Simon Sendler
 : @version 1.0
 : @since 1.0
 : @see http://bdn-edition.de/ns/digital
 : TODO: cleanup namespaces and imports
 : TODO: update namespaces to something project-specific
 :)
module namespace transformKMG="http://mollenhauer-edition.de/ns/transform";

import module namespace config="https://sade.textgrid.de/ns/config" at "../config.xqm";
import module namespace kmg-util="http://mollenhauer-edition.de/ns/util" at "kmg-util.xqm";
import module namespace bibref="https://mollenhauer-edition.de/ns/conversions/bibref" at "conversions/bibref.xqm";
import module namespace functx="http://www.functx.com";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace html="http://www.w3.org/1999/xhtml";
declare namespace tg="http://textgrid.info/namespaces/metadata/core/2010";
declare namespace m="http://www.w3.org/1998/Math/MathML";
declare default element namespace "http://www.w3.org/1999/xhtml";

declare boundary-space strip;

(:~
 : The main transformation routine. It iterates all nodes of a given TEI-XML
 : document and attempts to initiate transformation.
 : Should transformation fail, a span-Element will be included in the return,
 : including the error's code, description, as well as the node that raised
 : the exception.
 :
 : @param $nodes The nodes of a given TEI/XML document
 : @param $filename e.g. "2228v.xml"
 : @param $mode The edition view being transformed. "Digital" for interactive
 : view, for single view, the xml:id of the respective witness
 : @return The respective HTML nodes
 :)
declare function transformKMG:transform(
    $nodes as node()*, $filename as xs:string, $mode as xs:string, $witLeittext as xs:string, $listWit as xs:string*)
as node()*
{

    for $node in $nodes
    return
            (if ($node/@xml:id) then
                element span {
                    attribute class {
                        if ($node[self::tei:head][@type="ToC"]) then
                            "toc-anchor"
                        else
                            "text-anchor"
                    },
                    attribute id {
                        (if ($node[self::tei:head][@type="ToC"]) then
                            "toc-"
                        else
                            "")
                        || $node/@xml:id },
                    if ($node[self::tei:head][@type="ToC"]) then
                        attribute data-title {
                            transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
                        }
                    else
                        ()
                }
            else
                (),
            if ($node/@class = "error-message") then
                $node
            else
                try {
                    local:main($node, $filename, $mode, $witLeittext, $listWit)
                } catch * {
                    util:log-system-out(util:system-time() || " " || $filename || ": Error during transformation for " || $mode || "."),
                    element span {
                        attribute class {"error-message"},
                        "Error: ", $err:code, ". ", $err:description,
                        element br {},
                        "Offending node: ", functx:path-to-node-with-pos($node)
                    }
                })
};

declare function transformKMG:transform(
    $nodes as node()*, $filename as xs:string, $mode as xs:string)
as node()*
{
    let $listWit := kmg-util:getWitnessIDs($filename)
    let $witLeittext := kmg-util:getLeittextID($filename)
    return transformKMG:transform($nodes, $filename ,$mode ,$witLeittext ,$listWit)
};

(:~
 : The main transformation routine. It takes a TEI-xml node and transforms it
 : to a HTML node.
 :
 : For easier findability, elements in the typeswitch are sorted alphabetically.
 :
 : @param $nodes A TEI-xml node
 : @param $filename A TG-URI, e.g. "2228v.xml"
 : @param $mode The edition view being transformed.
 : @return the respective HTML nodes
 :)
declare function local:main(
    $node as node(), $filename as xs:string, $mode as xs:string, $witLeittext as xs:string, $listWit as xs:string*)
{
    typeswitch ($node)

        case text() return
            (: don't display any text between tei:rs and tei:u, but show any tei:rs with a reference :)
            if ($node[self::tei:u]/preceding-sibling::*[1][self::tei:rs[not(starts-with(@ref, "gnd"))]]) then
                ()
            else if ($node/following-sibling::*[1][self::tei:note]) then
                let $node := $node => replace("\n", "")
                return
                    text { $node }
            else if (matches($node, "\w$") and $node/following-sibling::node()[1][self::tei:table or self::tei:figure]) then
                text { $node || "&#0014;" }
            else
                text { $node }

        case comment() return
            ()

        case element (tei:anchor)
        return
            element span {
                attribute class {
                    "transcript--anchor transcript--anchor-" ||
                    (if ($node/@xml:id => matches("s$")) then
                        "start"
                    else
                        "end")
                },
                attribute id { $node/@xml:id }
            }

        case element (tei:app) return
            if (matches($mode, "-rdg$")) then
                transformKMG:transform($node/child::tei:rdg, $filename, $mode, $witLeittext, $listWit)
            else if ( $mode = "digital" ) then
                transformKMG:make-interactive-apparatus($node, $filename, $mode, $witLeittext, $listWit)
            else
                transformKMG:transform($node/child::*[contains(@wit, $mode)], $filename, $mode, $witLeittext, $listWit)

        case element (tei:bibl) return
            (transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit),
            for $token in $node/tokenize(@corresp, " ") return
                if (matches($token, "kmglit:")) then
                    let $clean-token := replace($token, "kmglit:", "")
                    let $content := bibref:make-listbibl($clean-token) => bibref:make-HTML()
                    return
                        (element span {
                            attribute class
                            {
                                "editorial-icon internal-reference js-editorial-info--bibl-internal"
                            },
                            attribute data-target {$token},
                            element span {
                                (attribute class {"fas fa-book"})
                            }
                        },
                        element div {
                            attribute data-ref {$token},
                            attribute class {
                                "editorial-content hidden internal-reference"
                            },
                            $content
                        })
                else
                    let $zoteroID := $token
                    return
                        if ($zoteroID = "") then
                            ()
                        else
                            element span {
                                attribute data-target {$zoteroID},
                                if (fn:exists($node/tei:citedRange)) then (
                                    attribute type {"citedRange"},
                                    attribute unit {$node/tei:citedRange/@unit},
                                    attribute from {$node/tei:citedRange/@from},
                                    attribute to {$node/tei:citedRange/@to}
                                )
                                else (),
                                attribute class
                                {
                                    "editorial-icon js-editorial-info--bibl" ||
                                    (if ($node/@type = "KMG") then
                                        "-KMG bibliographic-reference-added"
                                    else
                                        " bibliographic-reference"),
                                    (if ($node/parent::tei:cit/@corresp or $node/parent::tei:cit/@next or $node/parent::tei:cit/@prev
                                    or $node/parent::tei:q/@corresp or $node/parent::tei:q/@next or $node/parent::tei:q/@prev
                                    or $node/parent::tei:quote/@corresp or $node/parent::tei:quote/@next or $node/parent::tei:quote/@prev) then
                                        " linked-quote-reference"
                                    else
                                        ()
                                    )
                                },
                                attribute data-checked {
                                    if ($node/ancestor::tei:cit) then
                                        "true"
                                    else
                                        "false"
                                },
                                text { "&#8288;" },
                                element span {
                                    if ($node/@type = "KMG") then
                                        (attribute class {"fas fa-book-medical"})
                                    else
                                        (attribute class {"fas fa-book"})
                                }
                            }
            )

        case element (tei:cb) return
            let $prefix :=
                if ($node/preceding-sibling::*[1][self::tei:pb]) then
                    ""
                else
                    "|"
            return
                element span {
                    attribute class {"editorial"},
                    $prefix || $node/@n || "|"
                }

        case element (tei:cell)
        return
            element td
            {
                if ( $node/@rows ) then
                    attribute rowspan { $node/@rows }
                else
                    (),
                if ( $node/@cols ) then
                    attribute colspan { $node/@cols }
                else
                    (),
                if ( $node/@rend ) then
                    attribute class { $node/@rend }
                else
                    (),
                transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
            }

        case element (tei:choice)
        return
            transformKMG:choice($node, $filename, $mode, $witLeittext, $listWit)

        case element (tei:cit)
        return
            let $linkage := transformKMG:link-corresponding-quotes($node)
            return
            if ($linkage) then
                element span {
                    attribute class {"linked-quote"},
                    attribute data-corresp { $linkage },
                    attribute id { "cit-" || $node/@xml:id },
                    transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
                }
            else
                element span {
                    transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
                }

        case element (tei:corr) return
            if ($node/node()) then
                transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
            else
                text {"&#xF8;"}

        case element (tei:div) return
            element div {
                attribute {"type"} {$node/@type},
                transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
            }

        case element (tei:figure) return
            element figure {
                if ($node/@type="graphic" and $node/@rend = "inline") then
                    attribute class {"image image-inline"}
                else (),
                transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
            }

        case element (tei:floatingText) return
            element div {
                attribute class { $node/@type },
                transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
            }

        case element (tei:formula) return
            if ($node/@notation = "mathml") then
                element m:math {
                    attribute display {"inline"},
                    $node/node()
                }
            else
                ()

        case element (tei:front) return
            element div {
                attribute class {"preface"},
                transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
            }

        case element (tei:gap) return
            if ( $node/@unit = "characters" ) then
                element i {
                    attribute class { "fas fa-ellipsis-h" }
                }
            else
                element p {
                    element i {
                        attribute class { "fas fa-ellipsis-h" }
                    }
                }

        case element (tei:graphic) return
            if (fn:exists($node/@url) and fn:not(fn:empty($node/@url))) then (
                let $target :=
                    if ($node/@mimeType="image/tiff") then "https://textgridlab.org/1.0/digilib/rest/IIIF/" || $node/@url || "/full/700,/0/native.jpg"
                    else "https://textgridlab.org/1.0/tgcrud-public/rest/" || $node/@url || "/data"
                let $text :=
                    if (fn:exists($node/ancestor::tei:figure//tei:figDesc)) then (
                        transformKMG:transform($node/ancestor::tei:figure//tei:figDesc/node(), $filename, $mode, $witLeittext, $listWit)
                    )
                    else ("")
                let $image-element :=
                    element img {
                        attribute src {$target},
                        attribute alt {normalize-space(string-join($text, ""))},
                        attribute title {normalize-space(string-join($text, ""))}
                    }
                return
                    if (fn:exists($node/@corresp)) then (
                        element span {
                            attribute data-target {$node/@corresp/string()},
                            attribute class {"edition-image entity-name name artname editorial-icon js-editorial-info--art"},
                            $image-element
                        })
                    else (
                        $image-element
                    )
            )
            else ()

        case element (tei:figDesc) return
            ()

        case element (tei:head) return (
            if ($node/@type="ToC") then
                ()
            else
                transformKMG:heading($node, $filename, $mode, $witLeittext, $listWit)
        )

        case element (tei:hi) return
            if ($mode = "commentary") then
                element i {
                    transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
                }
            else
                element span {
                    attribute class {
                        if (fn:exists($node/@rend)) then
                            $node/@rend/string()
                        else
                            transformKMG:getHiRendStyleFromHeader($filename, $mode)
                    },
                    transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
                }

        case element ( tei:item ) return
            transformKMG:make-list-item($node, $filename, $mode, $witLeittext, $listWit)

        case element (tei:l) return
            element span {
                attribute class {"line-group__line"},
                transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit),
                if ($node/following-sibling::tei:l) then
                    element br {}
                else
                    ()
            }

        case element (tei:label) return
            switch ($node/@type)
                case "head" return
                    element span {
                        attribute class {"heading__label"},
                        transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit),
                        "&#x0020;"
                    }
                case "list" return
                    element div {
                        attribute class {"col-1"},
                        transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
                    }
                case "note_anchor" return
                   element sup {
                    attribute class {"authorial-note__anchor"},
                        transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
                    }
                default return
                    transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)

        case element (tei:lb) return
            element br {}

        case element (tei:lem) return
            if ( $mode = "digital" ) then
                ()
            else
                transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)

        case element (tei:lg) return (
            element div {
                attribute class {
                    "line-group",
                    if ( $node/@rend ) then $node/@rend else ()
                },
                if ($node/@n) then transformKMG:paragraphNumber(string($node/@n))
                else (),
                transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
            }
        )

        case element (tei:list) return
            if ($node/@type = "abbildungstext") then
                ()
            else
                if ($node/@type = "numbered") then
                    element ol {
                        transformKMG:list($node, $filename, $mode, $witLeittext, $listWit)
                    }
                else
                    element ul {
                        transformKMG:list($node, $filename, $mode, $witLeittext, $listWit)
                    }

        case element ( tei:media )
        return
            element audio {
                attribute controls { () },
                attribute preload { "metadata" },
                element source {
                    attribute src {"https://textgridlab.org/1.0/tgcrud-public/rest/" || $node/@url || "/data"},
                    attribute type {$node/@mimeType}
                },
                element p {
                    $node/tei:desc/text(),
                    ". ",
                    element a {
                        attribute href {"https://textgridlab.org/1.0/tgcrud-public/rest/" || $node/@url || "/data"},
                        "Download " || substring-after($node/@mimeType, "/") || "."
                    }
                }
            }

        case element (tei:milestone) return
            switch ($node/@type)
                case "paragraph"
                return
                    (element span {
                        attribute class {"paragraph paragraph-milestone"}
                    },
                    if ($node/@n) then transformKMG:paragraphNumber(string($node/@n))
                    else ())
                case "pagebreak"
                return
                    let $edRef :=
                        if ($mode = "digital") then
                            $node/@edRef => replace("[# ]", "")
                        else
                            $mode
                    let $wits :=
                        element span {
                            attribute class {"pagebreak--wits"},
                            $edRef
                        }
                    let $content := "|" || $wits || $node/@n || "|"
                    return
                        if ($mode = "digital" or matches($node/@edRef, $mode)) then
                            (
                                element span {
                                    attribute id { replace($node/@edRef, "#", "") || $node/@n},
                                    attribute data-title { $node/@n},
                                    attribute class {"page-anchor"}
                                },
                                element span {
                                attribute class {"editorial pagebreak"},
                                $content
                            })
                        else
                            ()
                default
                return
                    ()

        case element (tei:name) return
            let $type := $node/@type
            let $classMap :=
                map {
                    "art": "artname",
                    "brand": "brandname",
                    "music": "musicname",
                    "org": "orgname",
                    "periodica": "periodname",
                    "person": "persname"
                }
            return
                if ($type = "music") then
                    let $id := util:hash(generate-id($node), "md5")
                    let $target := "#" || $id
                    return
                        (element span {
                            attribute class {
                                "editorial-icon js-editorial-info--music",
                                if ($node/ancestor::tei:table) then
                                    "editorial-in-table"
                                else
                                    ""
                            },
                            attribute data-target {$id},
                            attribute data-key
                            {
                                if ( $node/@key ) then
                                    $node/@key
                                else
                                    $node//text()
                            },
                            transformKMG:transform($node/node()[not(self::tei:ref)], $filename, $mode, $witLeittext, $listWit)
                        },
                        element div {
                            attribute data-ref {$id},
                            attribute data-target {$node/@ref},
                            attribute class {"editorial-content hidden entity musical-work"},
                            attribute data-title { $node/@key => normalize-space()},
                            attribute data-source { $node/tei:ref/@target/string() },
                            transformKMG:transform($node/tei:ref, $filename, $mode, $witLeittext, $listWit)
                        }
                        )
                else if (not($node/@ref)) then
                    transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
                else
                element span {
                    if ($node/@ref or $node/@key) then
                        attribute class {"entity-name name " || $classMap($type) || " editorial-icon js-editorial-info--" || $type}
                    else
                        (),
                    if ($node/@ref) then
                        attribute data-target {
                            if (starts-with($node/@ref, "#")) then
                                transformKMG:resolve-reference($node)
                            else if (starts-with($node/@ref, "textgrid:46ntv")) then
                                $node/@ref => substring-after("#")
                            else
                                $node/@ref
                        }
                    else(),
                    transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit),
                    text { "&#8288;" }
                }

        case element (tei:note) return
            if ($node/@type="commentary") then
                let $id := util:hash(generate-id($node), "md5")
                let $target := "#" || $id
                return
                    (element span {
                        attribute class { "editorial-icon js-editorial-info--note" },
                        attribute data-target {$id},
                        element i {
                            attribute class {"fas fa-comment-alt"}
                        }
                    },
                    element div {
                        attribute data-ref {$id},
                        attribute class {"editorial-content hidden note"},
                        transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit),
                        if ($node/@resp) then
                            (
                            element span
                            {
                                attribute class {
                                    "entity-name name persname editorial-icon js-editorial-info--person"
                                },
                                attribute data-target {
                                    transformKMG:resolve-reference($node)
                                },
                                concat(" [", transformKMG:resolve-name($node),"]")
                            })
                        else
                            ()
                    })
            else
                (if ($node/@anchored = "false") then
                    ()
                else
                    if ($node/tei:app/*/tei:label[@type="note_anchor"]) then
                        let $note-variant :=
                            if ($mode = "digital") then
                                $node/tei:app/*[matches(@wit, $witLeittext)]/tei:label[@type="note_anchor"]
                            else
                                $node/tei:app/*[matches(@wit, $mode)]/tei:label[@type="note_anchor"]
                        return
                            transformKMG:transform($note-variant, $filename, $mode, $witLeittext, $listWit)
                    else if ($node//tei:label[@type="note_anchor"][not(tei:app)] or $mode != "digital") then
                        transformKMG:transform($node//tei:label[@type="note_anchor"], $filename, $mode, $witLeittext, $listWit)
                    else if ($node/tei:label[@type="note_anchor"]/tei:app) then
                        transformKMG:duplicate-apparatus($node/tei:label[@type="note_anchor"]/tei:app, $filename, $mode, $witLeittext, $listWit)
                    else
                        transformKMG:duplicate-apparatus($node/tei:app[descendant::tei:label[@type="note_anchor"]], $filename, $mode, $witLeittext, $listWit),
                    if (matches($node/ancestor::tei:note[1]/following-sibling::node(), "^\s")) then
                        ()
                    else
                        text { "&#8288;" },
                element div {
                    attribute class {"authorial-note"},
                    element button {
                        attribute class {"authorial-note-toggle"},
                        element span {
                            attribute class {"authorial-note-extend fas fa-chevron-left"}
                        }
                    },
                    element div {
                        attribute class {"authorial-note__content"},
                        transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
                    }
                })

        case element (tei:p) return
            (if ($node/ancestor::tei:figure) then
                element figcaption {
                    transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
                }
            else
                element div {
                    attribute class {
                        if ($node/parent::tei:item/ancestor::tei:list) then
                            "paragraph paragraph--list"
                        else if ($node/@rend = "small_letters") then
                            "paragraph small-letters"
                        else if ($node/@rend) then
                            "paragraph " || $node/@rend
                        else
                            "paragraph"
                    },
                    if ($node/@n) then transformKMG:paragraphNumber(string($node/@n))
                    else (),
                    transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
                }
            )

        case element (tei:pause)
        return
            element span {
                attribute class { "pause" },
                text { "&#x2002;–&#x2002;" }
            }

        case element (tei:pb) return
            let $wits := replace($node/@edRef, "#", "") => tokenize()
            let $mergedListOfWit := ($listWit,$wits)
            let $suffix :=
                if ($node/following-sibling::node()[1][self::tei:cb]) then
                    ""
                else
                    "|"
            let $editorialSpan :=
                if ($mode = "digital" or $mode = "digital-rdg") then
                    if (count($mergedListOfWit) != count(fn:distinct-values($mergedListOfWit))) then
                        (
                            element span {
                                    attribute id { replace($node/@edRef, "#", "") || $node/@n},
                                    attribute data-title { $node/@n},
                                    attribute class {"page-anchor"}
                                },
                            element span {
                                attribute class {"editorial pagebreak"},
                                "|",
                                element span {
                                    attribute class {"pagebreak--wits"},
                                    replace($node/@edRef, "#", "")
                                }, " " || $node/@n || $suffix
                            }
                        )
                    else ()
                else
                    if (contains($wits, $mode)) then
                        (
                            element span {
                                    attribute id { replace($node/@edRef, "#", "") || $node/@n},
                                    attribute data-title { $node/@n},
                                    attribute class {"page-anchor"}
                                },
                            element span {
                                attribute class {"editorial pagebreak"},
                                "|",
                                element span {
                                    attribute class {"pagebreak--wits"},
                                    $mode
                                }, " " || $node/@n || $suffix
                        })
                    else
                    ()
            return
                if ($node/ancestor::tei:table) then
                    element tr {
                        attribute class {"pagebreak-row"},
                        element td {
                            $editorialSpan
                        }
                    }
                else
                    $editorialSpan

        case element (tei:q) return
                transformKMG:make-quote($node, $filename, $mode, $witLeittext, $listWit)

        case element (tei:quote) return
                transformKMG:make-quote($node, $filename, $mode, $witLeittext, $listWit)

        case element (tei:ref) return
            if ($node/parent::tei:name/@type = "music") then
                element iframe {
                    if (matches($node/@target, "youtube")) then
                        attribute allow {
                            "accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture; web-share"
                        }
                    else
                        (),
                    attribute width {"100%"},
                    attribute height {"315"},
                    attribute loading {"lazy"},
                    attribute src {
                        if (matches($node/@target, "watch\?v=")) then
                            $node/@target => replace("watch\?v=", "embed/") || "?"
                        else
                            $node/@target
                    },
                    attribute title {
                        transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
                    }
                }
            else if (matches($node/@target, "^http")) then
                for $token in $node/tokenize(@target, " ") return
                    element a {
                        attribute href { $token },
                        attribute target { "_blank" },
                        transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit),
                        " ",
                        element span {
                            attribute class {"fas fa-external-link-alt"}
                        }
                    }
            else
            element span {
                attribute class {"reference"},
                    (transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit),
                    text { "&#8288;" },
                    for $token in $node/tokenize(@target, " ") return
                        if (starts-with($token, "#")) then
                            element a {
                                attribute class {
                                    "link--text",
                                    if ($mode = "digital" and $node/ancestor::tei:rdg) then
                                        "link--text--digital"
                                    else ()
                                    },
                                attribute href {
                                        $token
                                },
                                element i {
                                    attribute class {"fas fa-link"}
                                }
                            }
                        else
                            let $token-no-ns := $token => replace("textgrid:", "")
                            let $text :=  $token-no-ns => replace("[#\-].*", "")
                            return
                            if (doc-available($config:tei || $text || ".xml")) then
                                let $view :=
                                    if (matches($token, "-")) then $token-no-ns => replace("(.*-|#.*)", "") else
                                    if (kmg-util:count-witnesses($text) > 1) then "digital" else kmg-util:getLeittextID($text || ".xml")
                                let $anchor := if (matches($token, "#")) then "#" || substring-after($token, "#") else ""
                                let $href := "kmg.html?file=" || $text || "&amp;edition=" || $view || $anchor
                                return
                                    element a {
                                        attribute class {
                                            "link--text",
                                            if ($mode = "digital" and $node/ancestor::tei:rdg) then
                                                "link--text--digital"
                                            else ()
                                        },
                                        attribute href { $href },
                                        element i {
                                            attribute class {"fas fa-link"}
                                        }
                                    }
                            else ()
                        )
            }

        case element ( tei:row )
        return
            element tr {
                transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
            }

        case element ( tei:rs )
        return
            if ($node/@type and $node/@ref) then
                let $type := $node/@type
            let $classMap :=
                map {
                    "art": "artname",
                    "brand": "brandname",
                    "music": "musicname",
                    "org": "orgname",
                    "periodica": "periodname",
                    "person": "persname"
                }
            return
                element span {
                    if ($node/@ref or $node/@key) then
                        attribute class {"name " || $classMap($type) || " editorial-icon js-editorial-info--" || $type}
                    else
                        (),
                    if ($node/@ref) then
                        attribute data-target {
                            if (starts-with($node/@ref, "#")) then
                                transformKMG:resolve-reference($node)
                            else
                                $node/@ref
                        }
                    else(),
                    transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit),
                    text { "&#8288;" }
                }
            else
                transformKMG:transcript($node, $filename, $mode, $witLeittext, $listWit)

        case element (tei:said)
        return
            element span {
                attribute class {
                    if (fn:exists($node/@rend)) then
                        $node/@rend/string()
                    else
                        ""
                },
                transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
            }

        case element (tei:seg) return
            let $id := util:hash(generate-id($node/tei:sic), "md5")
            let $target := "#" || $id
            return
                if ($node/@resp) then
                    (
                        if ($node/@rend) then
                            element span {
                                attribute class { $node/@rend },
                                transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
                            }
                        else transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit),
                        text { "&#8288;" },
                        element span
                        {
                            attribute class {
                                "editorial-icon js-editorial-info--corr",
                                if ($node/ancestor::tei:table) then
                                    "editorial-in-table"
                                else
                                    ""
                            },
                            attribute data-target {$id},
                            element span {
                                attribute class {"fas fa-plus"}
                            }
                        },
                        element div {
                            attribute data-ref {$id},
                            attribute class {
                                "editorial-content hidden ",
                                if ($node/@type = "quote_hi") then
                                    "addhighlight"
                                else
                                    "addition"
                            },
                            for $person in tokenize($node/@resp, " ")
                            return
                                (element span
                                {
                                    attribute class { "editorial-icon js-editorial-info--person" },
                                    attribute data-target {
                                        substring-after(transformKMG:resolve-reference-string($node, $person), ":")
                                    },
                                    transformKMG:resolve-name-from-ref($node, $person)
                                }, " ")
                        }
                    )
                else if ( $node/@type = "margin" and (not($node/ancestor::tei:rdg[@type = "varying-structure" ])) ) then
                    element aside {
                        attribute class { "margin" },
                        element span {
                            attribute class { "editorial-icon fa fa-chevron-right"}
                        },
                        transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit) }
                else if ($node/@rend) then
                    element span {
                        attribute class { $node/@rend },
                        transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
                    }
                else if ($node/@type) then
                    element span {
                        attribute class { $node/@type },
                        transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
                    }
                else
                    transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)

        case element ( tei:table )
        return
            element div
            {
                attribute class { "table" },
                if ($node/@n) then transformKMG:paragraphNumber(string($node/@n))
                else (),
                element table {
                    if ($node/@rend) then
                        attribute class { $node/@rend }
                    else
                        (),
                    transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
                }
            }

        case element (tei:teiHeader) return
            ()

        case element (tei:text) return
            if ($mode = "commentary") then
                if ($node/@type = "commentary") then
                    element div {
                        transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
                    }
                else ()
            else
                if ($node/@type = "commentary") then ()
                else
                    element div {
                        transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
                    }

        case element (tei:trailer) return
            element tfoot {
                element tr {
                    element td {
                        attribute colspan { "99" },
                        transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
                    }
                }
            }

        case element (tei:u)
        return
            ()

        default return
            transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
};

declare function transformKMG:link-corresponding-quotes($node as node()) {
    if ($node/@corresp or $node/@next or $node/@prev) then
        let $corresp := if ($node/@corresp) then $node/@corresp else ()
        let $next := if ($node/@next) then $node/@next else ()
        let $prev := if ($node/@prev) then $node/@prev else ()
        let $correspIDs := string-join(($corresp, $next, $prev), " ") => replace("#", "")
        return
            $correspIDs
    else
        ()
};

(:~
 : Creates the correct kind of quotation marks around a quote
 :
 : @param $node A TEI-node containing a quotation
 : @param $filename E.g. "1234v.xml"
 : @param $mode The edition view being transformed.
 : @return A quotation
 :)
declare function transformKMG:make-quote(
    $node as node(), $filename as xs:string, $mode, $witLeittext as xs:string, $listWit as xs:string*)
as element(q)
{
    let $content := transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
    let $quote-marks := kmg-util:quote-marks($node/@rend, $filename, $mode)
    let $linkage := transformKMG:link-corresponding-quotes($node)
    let $quote :=
        ($quote-marks("open"), $content, $quote-marks("close"))
    return
        if ($node/@type = "block") then
            element div {
                if ($linkage) then
                    (attribute class {"linked-quote"},
                    attribute data-corresp { $linkage },
                    attribute id { "cit-" || $node/@xml:id })
                else
                    (),
                $quote
            }
        else
            element span {
                if ($linkage) then
                    (attribute class {"linked-quote"},
                    attribute data-corresp { $linkage },
                    attribute id { "cit-" || $node/@xml:id })
                else
                    (),
                $quote
            }
};

declare function transformKMG:list(
    $node as node(), $filename as xs:string, $mode, $witLeittext as xs:string, $listWit as xs:string*)
{
    (if ($node/parent::tei:item) then
        attribute class { "list list--nested" || $node/@rend }
    else if ($node/ancestor::tei:rdg) then
        attribute class {"list list--rdg" || $node/@rend }
    else if ($node/@rend) then
        attribute class {"list " || $node/@rend }
    else
        attribute class {"list" }
    ),
    attribute role {"list"},
    if ($node/child::tei:label) then
        for $item in $node/tei:item | $node/tei:app | $node/tei:pb return
        element li {
            attribute class {
                if ($item/@rend) then
                    "row gx-2 list-item " || replace($item/@rend, "_", "-")
                else
                    "row gx-2 list-item"
            },
            if ($item[self::tei:item]) then
                transformKMG:transform($item/preceding-sibling::tei:label[1], $filename, $mode, $witLeittext, $listWit)
            else
                (),
            transformKMG:transform($item, $filename, $mode, $witLeittext, $listWit)
        }
    else
        transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
};

(:~
 : Combines a tei:item and the preceding tei:label into one
 : list item
 :
 : @param $node A tei:item
 : @param $filename E.g. "1234v.xml
 : @param $mode The edition view being transformed.
 : @return A list item
 :)
declare function transformKMG:make-list-item(
    $node as node(), $filename as xs:string, $mode, $witLeittext as xs:string, $listWit as xs:string*)
as node()*
{
    if ($node/child::tei:label) then
        element li {
            attribute class {
                if ($node/@rend) then
                    "row gx-2 list-item " || replace($node/@rend, "_", "-")
                else
                    "row gx-2 list-item"
            },
            if ($node/@n) then transformKMG:paragraphNumber(string($node/@n))
            else (),
            transformKMG:transform($node/tei:label, $filename, $mode, $witLeittext, $listWit),
            element div {
                attribute class {"col-11"},
                transformKMG:transform($node/node()[not(self::tei:label)], $filename, $mode, $witLeittext, $listWit)
            }
        }
    else
        element div {
            attribute class {
                if ($node/@rend) then
                    "col-11 " || replace($node/@rend, "_", "-")
                else
                    "col-11"
            },
            if ($node/@n) then transformKMG:paragraphNumber(string($node/@n))
            else (),
            transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
        }
};

(:~
 : This function transforms a tei:app. Initiates the transformation of the
 : lemma, as well as any readings.
 :
 : Keep in mind that the produced readings are moved before storage.
 :
 : @param $node A tei:app
 : @param $filename E.g. "1234v.xml"
 : @param $mode The edition view being transformed.
 : @return The apparatus, transformed for HTML-presentation
 :)
declare function transformKMG:make-interactive-apparatus(
    $node as node(), $filename as xs:string, $mode as xs:string, $witLeittext as xs:string, $listWit as xs:string*)
as node()*
{
    if ($node/tei:lem/count(node()[normalize-space(.)]) = 0
        and $node/tei:rdg/count(node()[normalize-space(.)]) = 1 and $node/tei:rdg/tei:note[@type="editorial"])
    then
        ()
    else
        if (
            (: reading empty or contains only toc-head :)
            (
                $node/tei:rdg/count(node()[normalize-space(.)]) = 0 or
                ($node/tei:rdg/count(node()[normalize-space(.)]) = 1 and $node/tei:rdg/tei:head[@type="ToC"])
            )
            and
            (: lemma empty or contains only toc-head :)
            (
                $node/tei:lem/count(node()[normalize-space(.)]) = 0 or
                ($node/tei:lem/count(node()[normalize-space(.)]) = 1 and $node/tei:lem/tei:head[@type="ToC"])
            )
        ) then
            if ($node/node()[matches(@wit, $witLeittext)]/tei:head[@type="ToC"]) then
                transformKMG:transform($node/tei:*[matches(@wit, $witLeittext)]/tei:head[@type="ToC"], $filename, $mode, $witLeittext, $listWit)
            else
                ()
        (: in case there are only variants not pertaining to the current edition :)
        else if (count($node/child::tei:rdg[matches(@wit, "[" || string-join($listWit) || "]")]) = 0) then
                transformKMG:transform($node/tei:lem/*, $filename, $mode, $witLeittext, $listWit)
        else
            let $readings := $node/child::tei:*[not(matches(@wit, $witLeittext))][matches(@wit, "[" || string-join($listWit) || "]")]
            let $lem := $node/child::tei:*[matches(@wit, $witLeittext)][matches(@wit, "[" || string-join($listWit) || "]")]
            let $readings-no := count($readings)
            return
                (transformKMG:make-interactive-lemma($lem, $filename, $mode, $witLeittext, $listWit),
                for $reading in $readings return
                    (transformKMG:make-reading($reading, $filename, $mode || "-rdg", $witLeittext, $listWit),
                    if ($readings-no gt 1 and $reading = $readings[last()]) then
                        ()
                    else
                        element div {
                            attribute class {"rdg-digital__divider"}
                        }
                    )
                )
};

(: Clone of the regular apparatus function in cases we know that an element needs to be transformed twice,
 : such as footnote anchors. Using this function is a quick way to avoid duplicate IDs without needing to
 : postprocess the digital view. :)
declare function transformKMG:duplicate-apparatus(
    $node as node(), $filename as xs:string, $mode as xs:string, $witLeittext as xs:string, $listWit as xs:string*)
as node()*
{
    let $readings := $node/child::tei:rdg
    let $readings-no := count($readings)
    let $reading-wits := string-join($readings/@wit, "") => replace("[# ]+", "") => replace(" ", "")
    let $wits-attribute := if ( string-length($reading-wits) > 1 ) then ( "multiple" ) else ( $reading-wits )
    let $lem-id := util:hash(generate-id($node/tei:lem), "md5") || "-dup"

    return
        ((: in case there are only variants not pertaining to the current edition :)
        if (count($node/child::tei:rdg[matches(@wit, $witLeittext)][matches(@wit, "[" || string-join($listWit) || "]")]) = 0) then
            transformKMG:transform($node/tei:lem/node(), $filename, $mode, $witLeittext, $listWit)
        else
        element span {
            attribute class { "lemma" },
            attribute id {$lem-id},
            if ($reading-wits) then
                attribute wits {$reading-wits}
            else
                (),
            transformKMG:transform($node/tei:lem/node(), $filename, $mode, $witLeittext, $listWit)
        },
        for $reading in $readings
        let $rdg-id := util:hash(generate-id($reading), "md5") || "-dup"
        return
            (element dl {
                attribute class
                {
                    "rdg-digital rdg-digital--" || $wits-attribute,
                    if ( $reading/@type = "varying-structure" ) then
                        " varying-structure"
                    else
                        ""
                },
                attribute id {$rdg-id},
                attribute target-lem {"#" || $lem-id},
                attribute wits {$wits-attribute},
                element dt {
                    attribute class {"rdg-digital__siglum"},
                    element strong {$reading-wits}
                },
                element dd {
                    attribute class {
                        "rdg-digital__content"
                    },
                    if ($reading/node()) then
                        transformKMG:transform($reading/node(), $filename, $mode, $witLeittext, $listWit)
                    else
                        element span {"&#xF8;"}
                }
            },
            if ($readings-no gt 1 and $reading = $readings[last()]) then
                ()
            else
                element div {
                    attribute class {"rdg-digital__divider"}
                }
            )
        )
};

(:~
 : This function transforms a tei:lem.
 :
 : @param $node A tei:lem
 : @param $filename E.g. "1234v.xml"
 : @param $mode The edition view being transformed.
 : @return A lemma, transformed for HTML presentation
 :)
declare function transformKMG:make-interactive-lemma(
    $node, $filename, $mode, $witLeittext as xs:string, $listWit as xs:string*)
as node()*
{
        let $reading := $node/following-sibling::tei:* | $node/preceding-sibling::tei:*
        let $reading-wits := string-join($reading/@wit, "") => replace("[# ]+", "") => replace(" ", "")
        let $id := util:hash(generate-id($node/parent::*), "md5")
        return
            if ($node/node()) then
                if ($node//tei:div or $node//tei:p or $node//tei:lg or $node//tei:list or $node//tei:figure or $node//tei:graphic or $node/tei:item or $node/tei:head) then
                    (element div {
                        attribute class
                        {
                            "lemma lemma--block",
                            if ($node/tei:item) then
                                " row gx-2"
                            else ()
                        },
                        attribute id {$id},
                        if ($reading-wits) then
                            attribute wits {$reading-wits}
                        else
                            (),
                        transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
                    })
                else
                    (element span {
                        attribute class
                        {
                            "lemma"
                        },
                        attribute id {$id},
                        if ($reading-wits) then
                            attribute wits {$reading-wits}
                        else
                            (),
                        transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
                    })
            else
                element span {
                    attribute class
                    {
                        "lemma lemma--insertion"
                    },
                    attribute id {$id},
                    if ($reading-wits) then
                        attribute wits {$reading-wits}
                    else
                        (),
                    element sup {$reading-wits},
                    text {"&#x221A;"}
                }
};

(:~
 : Transforms a tei:rdg into a span containing the reading, links it
 : to the corresponding lemma.
 :
 : @param $reading A tei:rdg
 : @param $filename E.g. "1234v.xml"
 : @param $mode The edition view being transformed.
 : @return A sequence of span
 :)
declare function transformKMG:make-reading(
    $reading as node(), $filename as xs:string, $mode, $witLeittext as xs:string, $listWit as xs:string*)
as element()*
{
    let $wits := replace($reading/@wit, "[#]+", "")
    let $wits := replace($wits, " ", "")
    let $wits-attribute := if ( string-length($wits) > 1 ) then ( "multiple" ) else ( $wits )

    let $reading-id := util:hash(generate-id($reading), "md5")
    let $lemma-id := util:hash(generate-id($reading/parent::*), "md5")
    let $target-lemma := "#" || $lemma-id
    return
        element dl {
            attribute class
            {
                "rdg-digital rdg-digital--" || $wits-attribute,
                if ( $reading/@type = "varying-structure" ) then
                    " varying-structure"
                else
                    ""
            },
            attribute id {$reading-id},
            attribute target-lem {$target-lemma},
            attribute wits {$wits-attribute},
            element dt {
                attribute class {"rdg-digital__siglum"},
                element strong {$wits}
            },
            element dd {
                attribute class {
                    "rdg-digital__content"
                },
                if ($reading/node()) then
                    transformKMG:transform($reading/node(), $filename, $mode, $witLeittext, $listWit)
                else
                    element span {"&#xF8;"}
            }
        }
};

(:~
 : Resolves an abbreviated reference, adding a base-url to an id.
 :
 : @param $node A node containing an @corresp, @ref, @target, or @resp
 : @return A string containing the full reference
 :)
declare function transformKMG:resolve-reference($node as node())
as xs:string?
{
    $node/root()//*[@xml:id = replace($node/attribute()[name() = ('corresp', 'ref', 'target', 'resp')], '#', '')]/@ref/string()
};

declare function transformKMG:resolve-reference-string($node as node(), $ref as xs:string)
as xs:string?
{
    $node/root()//*[@xml:id = replace($ref, '#', '')]/@ref/string()
};

(:~
 : Retrieves a name that is referenced elsewhere in the document.l
 :
 : @param $node A node containing an @corresp, @ref, @target, or @resp
 : @return A string containing the resolved name
 :)
declare function transformKMG:resolve-name($node as node())
as xs:string?
{
  let $reference := $node/root()//*[@xml:id = replace($node/attribute()[name() = ('corresp', 'ref', 'target', 'resp')], '#', '')]
    return
        string-join(($reference/*[text()]), " ")
};

declare function transformKMG:resolve-name-from-ref($node as node(), $ref as xs:string)
as xs:string?
{
  let $reference := $node/root()//*[@xml:id = replace($ref, '#', '')]
    return
        string-join(($reference/*[text()]), " ")
};

declare function transformKMG:choice($node, $filename, $mode, $witLeittext as xs:string, $listWit as xs:string*)
{
    let $id := util:hash(generate-id($node/*[1]), "md5")
    let $target := "#" || $id
    let $text := element span {
            if ($node/tei:sic/node()) then
                transformKMG:transform($node/tei:sic, $filename, $mode, $witLeittext, $listWit)
            else if ( $node/tei:abbr ) then
                transformKMG:transform($node/tei:abbr, $filename, $mode, $witLeittext, $listWit)
            else
                element span {"&#xF8;"},
            element span {
                attribute class
                {
                    "editorial-icon",
                    if ( $node/tei:sic ) then
                        "js-editorial-info--corr correction"
                    else
                        "js-editorial-info--abbr abbreviation",
                    if ($node/ancestor::tei:table) then
                        "editorial-in-table"
                    else
                        ""
                },
                attribute data-target {$id},
                element span {
                    attribute class {
                        "fas ",
                        if ( $node/tei:abbr ) then
                            "fa-expand-alt"
                        else
                            "fa-pen"
                    }
                }
            }
        }
    let $modalcontent :=
        element div {
            attribute data-ref {$id},
            attribute class {
                "editorial-content hidden ",
                if ( $node/tei:abbr ) then
                    "abbreviation"
                else
                    "correction"
            },
            if ( $node/tei:corr ) then
                (
                if ($node/ancestor::*/@rend) then
                    element span {
                        attribute class { $node/ancestor::*/@rend/string() },
                        transformKMG:transform($node/tei:corr, $filename, $mode, $witLeittext, $listWit)
                    }
                else
                    transformKMG:transform($node/tei:corr, $filename, $mode, $witLeittext, $listWit),
                if ($node/tei:corr/@resp) then
                    (" ",
                    element span
                    {
                        attribute class {
                            "name editorial-icon js-editorial-info--person"
                        },
                        attribute data-target {
                            transformKMG:resolve-reference($node/tei:corr)
                        },
                        concat("[", transformKMG:resolve-name($node/tei:corr),"]")
                    })
                else
                    ())
            else
                transformKMG:transform($node/tei:expan, $filename, $mode, $witLeittext, $listWit)
        }
    return
        ($modalcontent, $text)
};

(:~
 : This function checks which @rend styles in the corresponding Witness from
 : the header are relevant for the tei:hi element here.
 : In the "digital" mode, the guide text is selected.
 : For all others, the corresponding single text, e.g. "a".
 :
 : @param $filename The filename of the XML file to be transformed. e.g. "2228v.xml"
 : @param $mode The edition view being transformed. "Digital" for interactive
 : view, for single view, the xml:id of the respective witness
 : @return A xs:string element that contains one or multiple rend styles.
 :)
declare function transformKMG:getHiRendStyleFromHeader($filename as xs:string, $mode as xs:string) as xs:string{
    let $doc := doc($config:tei || $filename)
    let $listWit := $doc//tei:teiHeader/tei:fileDesc/tei:sourceDesc/tei:listWit
    return
        if ($mode = "digital") then
            if (fn:exists($listWit//tei:desc[contains(@type, 'Leittext')])) then
                transformKMG:getMatchingHiRendStylesFromWitness(
                    $listWit//tei:desc[contains(@type, 'Leittext')]/../@rend/string())
            else ("no-Leittext-style-information-found")
        else if (fn:exists($listWit//tei:witness[@xml:id = $mode])) then
            transformKMG:getMatchingHiRendStylesFromWitness(
                $listWit//tei:witness[@xml:id = $mode]/@rend/string())
        else "no-valid-style-information-found"
};

(:~
 : This function checks if the passed string contains @rend styles that belong to
 : the set of styles valid in the metadata schema.
 : If there are more than one, a concatenated string is passed with a space as separator.
 :
 : @param $witnessRendStyles The list of rend styles that are in the corresponding witness of the header.
 : @return A xs:string element that contains one or multiple rend styles.
 :)
declare function transformKMG:getMatchingHiRendStylesFromWitness($witnessRendStyles as xs:string) as xs:string {
    let $hiRendStyles := ["body_type","bold","initial","italics","small_caps","spaced","under","versal"]
    return string-join(
        array:for-each($hiRendStyles, function($z) {
            if (contains($witnessRendStyles,$z)) then $z
            else ()
        })
    , " ")
};

declare function transformKMG:transcript($node as node()*, $filename as xs:string, $mode as xs:string, $witLeittext as xs:string, $listWit as xs:string*)
{
    let $utterance := $node/following-sibling::*[1][self::tei:u]
    return
        element div {
            attribute class {"row"},
            element div {
                attribute class { "col-1" },
                transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit),
                $node/following-sibling::text()[1],
                "&#x2002;"
            },
            element div {
                attribute class { "col-11 utterance" },
                if ($utterance/@start) then
                    (attribute data-utterance-start {$utterance/@start},
                    attribute data-utterance-end {$utterance/@end})
                else
                    (),
                transformKMG:transform($utterance/node(), $filename, $mode, $witLeittext, $listWit)
            }
        }
};

declare function transformKMG:heading($node as node()*, $filename as xs:string, $mode as xs:string, $witLeittext as xs:string, $listWit as xs:string*) {
    let $relevant-ancestors := count(
        $node/ancestor::tei:front
        | $node/ancestor::tei:table
        | $node/ancestor::tei:list
        | $node/ancestor::tei:div)
    let $sequence := if ($node/preceding-sibling::tei:head) then 2 else 1
    let $level := $relevant-ancestors + $sequence
    let $heading-level := if ($level > 6) then 6 else $level
    return
        element { "h" || $heading-level } {
            if ($node/ancestor::tei:table) then
                attribute class { "table-heading" }
            else
                (),
            transformKMG:transform($node/node(), $filename, $mode, $witLeittext, $listWit)
        }
};


(:~
 : This function creates a span to represent the paragraph Number
 :
 : @param $n The paragraph number as string
 : @return HTML span element
 :)
declare function transformKMG:paragraphNumber($n as xs:string){
    element span {
        attribute class {"editorial"},
        "[" || $n || "] "
    }
};
